#  SPDX-License-Identifier: MPL-2.0
#  Copyright 2020-2022 John Mille <john@compose-x.io>

from os import path
from pathlib import Path

from ecs_composex.mods_manager import XResourceModule

from .msk_cluster import MskCluster
from .msk_cluster_stack import XStack

COMPOSE_X_MODULES: dict = {
    "x-msk_cluster": {
        "Module": XResourceModule(
            "x-msk_cluster",
            XStack,
            Path(path.abspath(path.dirname(__file__))),
            MskCluster,
        ),
    }
}
